"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const errors_1 = require("./errors");
const addAGMA_1 = require("./addAGMA");
addAGMA_1.addApiGatewayManagementApi(aws_sdk_1.default);
class ConnectionNotFoundError extends errors_1.ExtendableError {
}
exports.ConnectionNotFoundError = ConnectionNotFoundError;
class DynamoDBConnectionManager {
    constructor({ connectionsTable = 'Connections' } = {}) {
        this.hydrateConnection = async (connectionId) => {
            // if connection is not found, throw so we can terminate connection
            const result = await this.db
                .get({
                TableName: this.connectionsTable,
                Key: {
                    id: connectionId,
                },
            })
                .promise();
            if (result.Item == null) {
                throw new ConnectionNotFoundError(`Connection ${connectionId} not found`);
            }
            return result.Item;
        };
        this.registerConnection = async ({ connectionId, endpoint, }) => {
            const connection = { id: connectionId, data: { endpoint } };
            await this.db
                .put({
                TableName: this.connectionsTable,
                Item: {
                    createdAt: new Date(),
                    id: connection.id,
                    data: connection.data,
                },
            })
                .promise();
            return connection;
        };
        this.sendToConnection = async (connection, payload) => {
            const { data: { endpoint }, id, } = connection;
            const managementApi = new aws_sdk_1.default.ApiGatewayManagementApi({
                endpoint,
                apiVersion: '2018-11-29',
            });
            try {
                await managementApi
                    .postToConnection({ ConnectionId: id, Data: payload })
                    .promise();
            }
            catch (e) {
                // this is stale connection
                // remove it from DB
                if (e && e.statusCode === 410) {
                    await this.unregisterConnection(connection);
                }
                else {
                    throw e;
                }
            }
        };
        this.unregisterConnection = async (connection) => {
            await this.db
                .delete({
                Key: {
                    id: connection.id,
                },
                TableName: this.connectionsTable,
            })
                .promise();
            // @todo delete all subscriptions too
        };
        this.connectionsTable = connectionsTable;
        this.db = new aws_sdk_1.default.DynamoDB.DocumentClient();
    }
}
exports.DynamoDBConnectionManager = DynamoDBConnectionManager;
exports.default = DynamoDBConnectionManager;
