"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// import AWS from 'aws-sdk';
const errors_1 = require("./errors");
const addAGMA_1 = require("./addAGMA");
const AWS = require('aws-sdk');
addAGMA_1.addApiGatewayManagementApi(AWS);
class ConnectionNotFoundError extends errors_1.ExtendableError {
}
exports.ConnectionNotFoundError = ConnectionNotFoundError;
class MemoryConnectionManager {
    constructor({ connectionsTable = 'Connections' } = {}) {
        this.hydrateConnection = async (connectionId) => {
            // if connection is not found, throw so we can terminate connection
            const result = this.db[this.connectionsTable + connectionId];
            if (result == null) {
                throw new ConnectionNotFoundError(`Connection ${connectionId} not found`);
            }
            return result;
        };
        this.registerConnection = async ({ connectionId, endpoint, }) => {
            const connection = { id: connectionId, data: { endpoint } };
            this.db[this.connectionsTable + connection.id] = {
                createdAt: new Date(),
                id: connection.id,
                data: connection.data,
            };
            return connection;
        };
        this.sendToConnection = async (connection, payload) => {
            const { data: { endpoint }, id, } = connection;
            const managementApi = new AWS.ApiGatewayManagementApi({
                endpoint,
                apiVersion: '2018-11-29',
            });
            try {
                await managementApi
                    .postToConnection({ ConnectionId: id, Data: payload })
                    .promise();
            }
            catch (e) {
                // this is stale connection
                // remove it from DB
                if (e && e.statusCode === 410) {
                    await this.unregisterConnection(connection);
                }
                else {
                    throw e;
                }
            }
        };
        this.unregisterConnection = async (connection) => {
            delete this.db[this.connectionsTable + connection.id];
            // @todo delete all subscriptions too
        };
        this.connectionsTable = connectionsTable;
        this.db = {};
    }
}
exports.MemoryConnectionManager = MemoryConnectionManager;
exports.default = MemoryConnectionManager;
